@@ -33,7 +33,6 @@ import org.springframework.context.ResourceLoaderAware;
 import org.springframework.core.io.DefaultResourceLoader;
 import org.springframework.core.io.Resource;
 import org.springframework.core.io.ResourceLoader;
-import org.springframework.util.Assert;
 import org.springframework.util.DefaultPropertiesPersister;
 import org.springframework.util.PropertiesPersister;
 import org.springframework.util.StringUtils;
@@ -58,14 +57,14 @@ import org.springframework.util.StringUtils;
  * prefix, resources can still be loaded from the classpath, but "cacheSeconds" values
  * other than "-1" (caching forever) might not work reliably in this case.
  *
- * <p>For a typical web application, message files could be placed into {@code WEB-INF}:
- * e.g. a "WEB-INF/messages" basename would fine a "WEB-INF/messages.properties",
+ * <p>For a typical web application, message files could be placed in {@code WEB-INF}:
+ * e.g. a "WEB-INF/messages" basename would find a "WEB-INF/messages.properties",
  * "WEB-INF/messages_en.properties" etc arrangement as well as "WEB-INF/messages.xml",
  * "WEB-INF/messages_en.xml" etc. Note that message definitions in a <i>previous</i>
  * resource bundle will override ones in a later bundle, due to sequential lookup.
 
  * <p>This MessageSource can easily be used outside of an
- * {@link org.springframework.context.ApplicationContext}: It will use a
+ * {@link org.springframework.context.ApplicationContext}: it will use a
  * {@link org.springframework.core.io.DefaultResourceLoader} as default,
  * simply getting overridden with the ApplicationContext's resource loader
  * if running in a context. It does not have any other specific dependencies.
@@ -85,23 +84,15 @@ import org.springframework.util.StringUtils;
  * @see ResourceBundleMessageSource
  * @see java.util.ResourceBundle
  */
-public class ReloadableResourceBundleMessageSource extends AbstractMessageSource implements ResourceLoaderAware {
+public class ReloadableResourceBundleMessageSource extends AbstractResourceBasedMessageSource implements ResourceLoaderAware {
 
 	private static final String PROPERTIES_SUFFIX = ".properties";
 
 	private static final String XML_SUFFIX = ".xml";
 
 
-	private String[] basenames = new String[0];
-
-	private String defaultEncoding;
-
 	private Properties fileEncodings;
 
-	private boolean fallbackToSystemLocale = true;
-
-	private long cacheMillis = -1;
-
 	private boolean concurrentRefresh = true;
 
 	private PropertiesPersister propertiesPersister = new DefaultPropertiesPersister();
@@ -121,72 +112,12 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 			new ConcurrentHashMap<Locale, PropertiesHolder>();
 
 
-	/**
-	 * Set a single basename, following the basic ResourceBundle convention of
-	 * not specifying file extension or language codes, but in contrast to
-	 * {@link ResourceBundleMessageSource} referring to a Spring resource location:
-	 * e.g. "WEB-INF/messages" for "WEB-INF/messages.properties",
-	 * "WEB-INF/messages_en.properties", etc.
-	 * <p>XML properties files are also supported: .g. "WEB-INF/messages" will find
-	 * and load "WEB-INF/messages.xml", "WEB-INF/messages_en.xml", etc as well.
-	 * @param basename the single basename
-	 * @see #setBasenames
-	 * @see org.springframework.core.io.ResourceEditor
-	 * @see java.util.ResourceBundle
-	 */
-	public void setBasename(String basename) {
-		setBasenames(basename);
-	}
-
-	/**
-	 * Set an array of basenames, each following the basic ResourceBundle convention
-	 * of not specifying file extension or language codes, but in contrast to
-	 * {@link ResourceBundleMessageSource} referring to a Spring resource location:
-	 * e.g. "WEB-INF/messages" for "WEB-INF/messages.properties",
-	 * "WEB-INF/messages_en.properties", etc.
-	 * <p>XML properties files are also supported: .g. "WEB-INF/messages" will find
-	 * and load "WEB-INF/messages.xml", "WEB-INF/messages_en.xml", etc as well.
-	 * <p>The associated resource bundles will be checked sequentially when resolving
-	 * a message code. Note that message definitions in a <i>previous</i> resource
-	 * bundle will override ones in a later bundle, due to the sequential lookup.
-	 * @param basenames an array of basenames
-	 * @see #setBasename
-	 * @see java.util.ResourceBundle
-	 */
-	public void setBasenames(String... basenames) {
-		if (basenames != null) {
-			this.basenames = new String[basenames.length];
-			for (int i = 0; i < basenames.length; i++) {
-				String basename = basenames[i];
-				Assert.hasText(basename, "Basename must not be empty");
-				this.basenames[i] = basename.trim();
-			}
-		}
-		else {
-			this.basenames = new String[0];
-		}
-	}
-
-	/**
-	 * Set the default charset to use for parsing properties files.
-	 * Used if no file-specific charset is specified for a file.
-	 * <p>Default is none, using the {@code java.util.Properties}
-	 * default encoding: ISO-8859-1.
-	 * <p>Only applies to classic properties files, not to XML files.
-	 * @param defaultEncoding the default charset
-	 * @see #setFileEncodings
-	 * @see org.springframework.util.PropertiesPersister#load
-	 */
-	public void setDefaultEncoding(String defaultEncoding) {
-		this.defaultEncoding = defaultEncoding;
-	}
-
 	/**
 	 * Set per-file charsets to use for parsing properties files.
 	 * <p>Only applies to classic properties files, not to XML files.
 	 * @param fileEncodings Properties with filenames as keys and charset
 	 * names as values. Filenames have to match the basename syntax,
-	 * with optional locale-specific appendices: e.g. "WEB-INF/messages"
+	 * with optional locale-specific components: e.g. "WEB-INF/messages"
 	 * or "WEB-INF/messages_en".
 	 * @see #setBasenames
 	 * @see org.springframework.util.PropertiesPersister#load
@@ -195,44 +126,12 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 		this.fileEncodings = fileEncodings;
 	}
 
-	/**
-	 * Set whether to fall back to the system Locale if no files for a specific
-	 * Locale have been found. Default is "true"; if this is turned off, the only
-	 * fallback will be the default file (e.g. "messages.properties" for
-	 * basename "messages").
-	 * <p>Falling back to the system Locale is the default behavior of
-	 * {@code java.util.ResourceBundle}. However, this is often not desirable
-	 * in an application server environment, where the system Locale is not relevant
-	 * to the application at all: Set this flag to "false" in such a scenario.
-	 */
-	public void setFallbackToSystemLocale(boolean fallbackToSystemLocale) {
-		this.fallbackToSystemLocale = fallbackToSystemLocale;
-	}
-
-	/**
-	 * Set the number of seconds to cache loaded properties files.
-	 * <ul>
-	 * <li>Default is "-1", indicating to cache forever (just like
-	 * {@code java.util.ResourceBundle}).
-	 * <li>A positive number will cache loaded properties files for the given
-	 * number of seconds. This is essentially the interval between refresh checks.
-	 * Note that a refresh attempt will first check the last-modified timestamp
-	 * of the file before actually reloading it; so if files don't change, this
-	 * interval can be set rather low, as refresh attempts will not actually reload.
-	 * <li>A value of "0" will check the last-modified timestamp of the file on
-	 * every message access. <b>Do not use this in a production environment!</b>
-	 * </ul>
-	 */
-	public void setCacheSeconds(int cacheSeconds) {
-		this.cacheMillis = (cacheSeconds * 1000);
-	}
-
 	/**
 	 * Specify whether to allow for concurrent refresh behavior, i.e. one thread
 	 * locked in a refresh attempt for a specific cached properties file whereas
 	 * other threads keep returning the old properties for the time being, until
 	 * the refresh attempt has completed.
-	 * <p>Default is "true": This behavior is new as of Spring Framework 4.1,
+	 * <p>Default is "true": this behavior is new as of Spring Framework 4.1,
 	 * minimizing contention between threads. If you prefer the old behavior,
 	 * i.e. to fully block on refresh, switch this flag to "false".
 	 * @since 4.1
@@ -273,7 +172,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	@Override
 	protected String resolveCodeWithoutArguments(String code, Locale locale) {
-		if (this.cacheMillis < 0) {
+		if (getCacheMillis() < 0) {
 			PropertiesHolder propHolder = getMergedProperties(locale);
 			String result = propHolder.getProperty(code);
 			if (result != null) {
@@ -281,7 +180,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 			}
 		}
 		else {
-			for (String basename : this.basenames) {
+			for (String basename : getBasenameSet()) {
 				List<String> filenames = calculateAllFilenames(basename, locale);
 				for (String filename : filenames) {
 					PropertiesHolder propHolder = getProperties(filename);
@@ -301,7 +200,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 */
 	@Override
 	protected MessageFormat resolveCode(String code, Locale locale) {
-		if (this.cacheMillis < 0) {
+		if (getCacheMillis() < 0) {
 			PropertiesHolder propHolder = getMergedProperties(locale);
 			MessageFormat result = propHolder.getMessageFormat(code, locale);
 			if (result != null) {
@@ -309,7 +208,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 			}
 		}
 		else {
-			for (String basename : this.basenames) {
+			for (String basename : getBasenameSet()) {
 				List<String> filenames = calculateAllFilenames(basename, locale);
 				for (String filename : filenames) {
 					PropertiesHolder propHolder = getProperties(filename);
@@ -339,8 +238,9 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 		}
 		Properties mergedProps = newProperties();
 		mergedHolder = new PropertiesHolder(mergedProps, -1);
-		for (int i = this.basenames.length - 1; i >= 0; i--) {
-			List<String> filenames = calculateAllFilenames(this.basenames[i], locale);
+		String[] basenames = StringUtils.toStringArray(getBasenameSet());
+		for (int i = basenames.length - 1; i >= 0; i--) {
+			List<String> filenames = calculateAllFilenames(basenames[i], locale);
 			for (int j = filenames.size() - 1; j >= 0; j--) {
 				String filename = filenames.get(j);
 				PropertiesHolder propHolder = getProperties(filename);
@@ -376,7 +276,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 		}
 		List<String> filenames = new ArrayList<String>(7);
 		filenames.addAll(calculateFilenamesForLocale(basename, locale));
-		if (this.fallbackToSystemLocale && !locale.equals(Locale.getDefault())) {
+		if (isFallbackToSystemLocale() && !locale.equals(Locale.getDefault())) {
 			List<String> fallbackFilenames = calculateFilenamesForLocale(basename, Locale.getDefault());
 			for (String fallbackFilename : fallbackFilenames) {
 				if (!filenames.contains(fallbackFilename)) {
@@ -447,7 +347,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 
 		if (propHolder != null) {
 			originalTimestamp = propHolder.getRefreshTimestamp();
-			if (originalTimestamp == -1 || originalTimestamp > System.currentTimeMillis() - this.cacheMillis) {
+			if (originalTimestamp == -1 || originalTimestamp > System.currentTimeMillis() - getCacheMillis()) {
 				// Up to date
 				return propHolder;
 			}
@@ -492,7 +392,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 	 * @param propHolder the current PropertiesHolder for the bundle
 	 */
 	protected PropertiesHolder refreshProperties(String filename, PropertiesHolder propHolder) {
-		long refreshTimestamp = (this.cacheMillis < 0 ? -1 : System.currentTimeMillis());
+		long refreshTimestamp = (getCacheMillis() < 0 ? -1 : System.currentTimeMillis());
 
 		Resource resource = this.resourceLoader.getResource(filename + PROPERTIES_SUFFIX);
 		if (!resource.exists()) {
@@ -501,7 +401,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 
 		if (resource.exists()) {
 			long fileTimestamp = -1;
-			if (this.cacheMillis >= 0) {
+			if (getCacheMillis() >= 0) {
 				// Last-modified timestamp of file will just be read if caching with timeout.
 				try {
 					fileTimestamp = resource.lastModified();
@@ -571,7 +471,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 					encoding = this.fileEncodings.getProperty(filename);
 				}
 				if (encoding == null) {
-					encoding = this.defaultEncoding;
+					encoding = getDefaultEncoding();
 				}
 				if (encoding != null) {
 					if (logger.isDebugEnabled()) {
@@ -631,7 +531,7 @@ public class ReloadableResourceBundleMessageSource extends AbstractMessageSource
 
 	@Override
 	public String toString() {
-		return getClass().getName() + ": basenames=[" + StringUtils.arrayToCommaDelimitedString(this.basenames) + "]";
+		return getClass().getName() + ": basenames=" + getBasenameSet();
 	}
 
 
